#set-executionpolicy unrestricted

<#
.SYNOPSIS
Backup VMs managed by a vsphere host
.DESCRIPTION
Backups all the VMs managed by a licensed vmware vsphere host
.PARAMETER  vsphere_host
FQDN or ip address of teh vmware host
.PARAMETER  vsphere_user 
User to access the esxi server
.PARAMETER  vsphere_password 
Password to access the esxi server
.PARAMETER  bk_repository 
The path of the backup files repository
.EXAMPLE .\veeamfreecbs.ps1
.NOTES 
NAME: VeeamFreeCompleteBackupScript
AUTHOR: Fabio Ribolla (fabio.ribolla@assyrus.it)  
KEYWORDS: Backup Veeam Free
#>

[CmdletBinding()]
param(
	#vSphere host parameters
	$vsphere_host = '192.168.116.137',
	$vsphere_user = 'root',
	$vsphere_password = 'Password09',
	
	#Backup parameters see http://helpcenter.veeam.com/backup/80/powershell/start-vbrzip.html
	$bk_repository = "E:\Veeam\Backup",
	#$bk_quiesce = "",
	#$bk_quiesce = "-DisableQuiesce"
	#Possible vlaues: Never, Tonight, TomorrowNight, In3days, In1Week, In2Weeks, In1Month
	$bk_autodelete = 'In1Week',
	#Possible values: 0,4,5,6,9 at higher values corresponds more compression and more cpu usage
	$bk_compression = 9,

	#Email report parameters
	$mail_send = "true",
	$mail_smtp = 'smtp.domain.com',
	$mail_sender = 'veeamfree@domain.com',
	$mail_recipient = 'recipient@domain.com',

	#Virtual machines names to exclude for the backup process
	$excludedVM = ("DC1", "VmName2", "VmName3")
)


if ( (Get-PSSnapin -Name VeeamPSSnapIn -ErrorAction SilentlyContinue) -eq $null )
{
    Add-PSSnapin VeeamPSSnapIn
}

if ( (Get-PSSnapin -Name VMware.VimAutomation.Core -ErrorAction SilentlyContinue) -eq $null )
{
    Add-PSSnapin VMware.VimAutomation.Core
}

Connect-VIServer -Server $vsphere_host -User $vsphere_user -Password $vsphere_password

$hosts = Get-VMHost
$hosts = $hosts | Sort-Object -property name

foreach($vmHost in $hosts)
{
	write-host "Backing up host: " $vmHost.name
	$result = Get-VMhost $vmHost.name | Get-VM | Where-Object {$_.Host.Name -eq $vmHost.name}
	$aBkResult = @()
	foreach($vm in $result)
	{
		if ($excludedVM  -notcontains $vm.name)
		{
			Write-Host "Backing up vm: $vm"
			$vm_tobackup = Find-VBRViEntity -Name $vm

			#BACKUP VM
			$bk_result = Start-VBRZip -Folder $bk_repository -Entity $vm_tobackup -Compression $bk_compression -autodelete $bk_autodelete
			$aBkResult = $aBkResult + $bk_result
		}
	}
}


###Create the mail report
$mail_text = $mail_text + "<tr align='center'><b><td>JOB NAME</td><td>START TIME</td><td>END TIME</td><td>DURATION (min.)</td><td>RESULT</td></b></tr>"
foreach($result in $aBkResult)
{
	$result_color = ""
	if ($result.result -eq "Success")
	{
		$result_color = "green"
	}
	elseif ($result.result -eq "Warning")
	{
		$result_color = "yellow"
	}
	elseif ($result.result -eq "Failed")
	{
		$result_color = "red"
	}
	$duration = new-timespan Start $result.CreationTime End $result.endtime

	$mail_text = $mail_text + "<tr><td>" +$result.JobName+ "</td><td>" +$result.CreationTime+ "</td><td>" +$result.endtime+ "</td><td>" +$duration.TotalMinutes+ "</td><td bgcolor='$result_color'>" +$result.result + "</td></tr>"
}

$mail_body = "<html>
<b>VeeamFree complete backup script report:<br /><br /></b>
<table border='1'>
$mail_text
</table>
<html>"
#####

#Send email report if enabled
if ($mail_send)
{
	$mail_result = send-mailmessage -from $mail_sender -to $mail_recipient -subject "VeeamFree backup report" -bodyashtml $mail_body -smtpServer $mail_smtp
}
###

#Write html report
$mail_body = $mail_body + $mail_result
$mail_body | Out-File $bk_repository\VeeamFreeCBSReport.html
###